/*------------------------------------------------------------------------------
-------------   BIBLIOTECA PARA MANIPULAO DE DISPLAYS LCD 16x2 --------------
                        PARA MSP430 - LAUNCHPAD da Texas
              ADAPTADA POR PROF. FRANCISCO FAMBRINI - FDI ELETRONICA
                                 ffambrini@gmail.com
              AGRADECIMENTOS AO PROF ALESSANDRO CUNHA - TECHTRAINING
        Para ser usada com MSP430G2231  da plaquinha LAUNCHPAD da Texas
                            Essa biblioteca  freeware
Use por sua conta e risco - O autor no se responsabiliza pelo no funcionamento
--------------------------------------------------------------------------------
            Plaquinha LaunchPad              display LCD 16x2
                    P1.2                        RS
                    P1.3                        E
                    P1.4                        D4
                    P1.5                        D5
                    P1.6                        D6
                    P1.7                        D7
                     GND                        GND          
    Alimentar o Modulo LCD com 5V externo a placa LaunchPad
    Ligar +5V no pino 2 do LCD porque o MSP430 trabalha com 3,6V
    aterrar R/W  do lcd (pino 5 do LCD no GND ).
    Essa biblioteca usa o pino R/W aterrado, nao usa o busy flag
    Ligar trim-pot de 10k no pino 3 de contraste do LCD e ajustar o contraste
--------------------------------------------------------------------------------*/
#define  E_0     P1OUT=P1OUT & 0xF7     // enable em nivel baixo   
#define  E_1     P1OUT=P1OUT | 0x08    //  enable em nivel alto 
#define  RS_0    P1OUT=P1OUT & 0xFB    //  rs em nivel baixo    
#define  RS_1    P1OUT=P1OUT | 0x04    //  rs em nivel alto
#define  D4_0    P1OUT=P1OUT & 0xEF
#define  D4_1    P1OUT=P1OUT | 0x1F
#define  D5_0    P1OUT=P1OUT & 0xDF
#define  D5_1    P1OUT=P1OUT | 0x2F
#define  D6_0    P1OUT=P1OUT & 0xBF
#define  D6_1    P1OUT=P1OUT | 0x4F
#define  D7_0    P1OUT=P1OUT & 0x7F
#define  D7_1    P1OUT=P1OUT | 0x8F

//  Variaveis Globais
    char disp[32];  // Buffer que armazena 32 caracteres para serem mostrados no LCD
     
 // prototipo das funoes globais
    void init_lcm(void);       // Funao que Inicializa o modulo LCD
    void putchar(char what);   // Funcao que imprime um caractere na posicao atual do cursor
    void putstr(char *what);   // Funcao que imprim uma string na posicao atual do cursor
    void clr_lcm(void);        // Funcao que limpa o display LCD
    void lcm_line2(void);      // Funcao que move o cursor para a segunda linha
    void delay(char);          // Funcao que gera os delays necessarios para o LCD
    void strobe(void);         // Funcao que gera o sinal de habilitacao para o pino E do display
    void update_lcm(char *str);// Funcao que escreve o ' disp[32] '  no display
                               // str  uma string de 32 caracteres
    void clr_disp_buffer(void);// Funcao que limpa disp[32] preenchendo-o com zeros
    void tostr(int i,char *str); //Converte inteiro para String e imprime o inteiro no LCD ate 999
    void lcd_pos_00( void );     //posiciona o cursor no inicio da primeira linha, sem limpar o lcd
 //-----------------------------------------------------------------------------    
    void tostr(int i,char *s)       // Converte Inteiro para String, ate 3 digitos, at 999
                                    // imprime o inteiro de 000 at 999 diretamente no LCD
    {
            char *p;
            p=s;
         
        if(i >= 900) 
                    p[0]= 9;
         else if(i >= 800) 
                    p[0]= 8; 
         else   if(i >= 700) 
                    p[0]= 7;
         else  if(i >= 600) 
                    p[0]= 6;
          else   if(i >= 500) 
                    p[0]= 5;
           else if(i >= 400) 
                     p[0]= 4;
           else if(i >= 300)
                    p[0]= 3;
           else if(i >= 200)
                    p[0]= 2;
            else if(i >= 100)
                    p[0]= 1;
            else
                    p[0]=0;
                   
            p[2]=i%10;
           
            i-=p[2];
            i/=10;
            p[1]=i%10;
           
            p[3] = 0; // mark end of string
            p[2]+=0x30;
            p[1]+=0x30;
            p[0]+=0x30;
  //-----------------------------------------
            //adaptado por FRANCISCO FAMBRINI
            // para escrever com 3 digitos mudar ind=1 para ind=0
            for (int ind=1; ind<3; ind++) {
                                            putchar(p[ind]);
                                          }
    }
  //----------------------------------------------------------------------------     
 void clr_disp_buffer(void)  { // Limpa o buffer interno, preenchendo-o com espaos vazios
      for(char x = 0; x <32; x++)  disp[x] = ' ';
    }
 //-----------------------------------------------------------------------------

//------------------------------------------------------------------------------    
    void strobe(void)
    { // Ativa o pino Enable do LCD quando necessario 
       E_0  ;               //Enable em nivel baixo
       delay(105);
       E_1  ;              //Enable em nivel alto
       delay(85);
    }
//------------------------------------------------------------------------------
    
//------------------------------------------------------------------------------
    void init_lcm(void)
    {
       P1DIR =  P1DIR |  0xFC;   
       P1OUT =  P1OUT  | 0xFC;   //Seta todos os bits, exceto P1.1 e P1.0
       
       // Ajusta o compimento da interface para LCD 16x2
       E_1 ; //coloca o pino Enable em nivel alto 
       P1OUT=P1OUT & 0xFB; //Seleciona o Modo de Comando do LCD
       P1OUT=0x2C;
       P1OUT=P1OUT & 0xFB; //Modo de Comando
       strobe();
       
       //  LCD trabalhando no modo de 4 bits
       //  Byte que sera transmitido sera quebrado em 2 partes
       //  Vamos transmitir o Nibble Alto (HN) seguido 
       //  do Nibble Baixo (LN)
       
        //Habilita o modo de 2 Linhas
       
       //HN
       P1OUT=0x2C;
       P1OUT=P1OUT & 0xFB; //Ajusta o modo de Comando
       strobe();
           
       //LN
       P1OUT=0x8C;
       P1OUT=P1OUT & 0xFB; //Ajusta Modo de Comando
       strobe();
         
       //Ajusta os Parametros do Cursor    
       //HN
       P1OUT=0x0C;
       P1OUT=P1OUT & 0xFB;
       strobe();
           
       //LN
       P1OUT=0xFC;
       P1OUT=P1OUT & 0xFB;
       delay(65);
       strobe();
          
       //Leva o Cursor do LCD para a posio inicial
       lcd_pos_00();
    }
 //-----------------------------------------------------------------------------
    void lcm_line2(void)
    {
       // Salta para a linha 2 do LCD
       //HN
       P1OUT=0xCC;
       P1OUT=P1OUT & 0xFB;
       strobe();        
       //LN
       P1OUT=0x0C;
       P1OUT=P1OUT & 0xFB;
       strobe();
    }
//------------------------------------------------------------------------------     
    void update_lcm(char *str)
    { // Escreve o conteudo de  "disp[32] buffer"   no LCD
      char pos;
      // escreve os primeiros 16 caracteres
      clr_lcm();
      for(pos = 0; pos <16; pos++ )
      {
         putchar(str[pos]);
      }
      // escreve os proximos 16 caracteres da 2 linha
      lcm_line2();// linha 2
      for(pos = 16; pos<32;pos++)
      {
        putchar(str[pos]);
      }
    }
 //-----------------------------------------------------------------------------
    void clr_lcm(void)
    {
       // Limpa o display LCD
       //HN
       P1OUT=0x0C;
       P1OUT=P1OUT & 0xFB;
       strobe();
           
       //LN
       P1OUT=0x1C;
       P1OUT=P1OUT & 0xFB;
       strobe();
       }
 //-----------------------------------------------------------------------------
    void putchar(char ascii)
    {     // Imprime um unico caractere na posio atual apontada pelo cursor
            // HN
            P1OUT=0x0C | (ascii & 0xFC ) ;
            delay(115);
            strobe();
            // LN
            P1OUT=0x0C | ( (ascii<<4 )& 0xFC  ); // Libera os 4 bits mais baixos 
                                                 // no barramento de dados de 4 bits do cursor
            strobe();
    }
 //-----------------------------------------------------------------------------    
    void putstr(char *str) {
      while (*str) { putchar(*str++);  }
    }
//-------------------------------------------------------------------------
    void delay(char d) { // funo usada para gerar delays
                               for(int i = 0; i <2*d;i++);
                       }
//----------------------------------------------------------------------------
 
 //-----------------------------------------------------------------------
 void lcd_pos_00( void ){
//Leva o Cursor do LCD para a posio inicial sem limpar o display
       // nibble alto
       P1OUT=0x0C;
       P1OUT=P1OUT & 0xFB;
       strobe();     
       // nibble baixo
       P1OUT=0x3C;
       P1OUT=P1OUT & 0xFB;
       strobe();   
   }
 //--------------------------------------------------------------------------

